package com.wtkj.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.wtkj.dto.UpdateUserDTO;
import com.wtkj.entity.ContactOuter;
import com.wtkj.service.IContactOuterService;
import com.wtkj.service.IWorkStatusService;
import com.wtkj.vo.WorkStatusVO;
import com.wtkj.wrapper.ContactOuterUserWrapper;
import com.wtkj.wrapper.ContactOuterWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * @author Blizzard
 * @create at 2023-09-20 11:58
 * @describe
 */
@RestController
@AllArgsConstructor
@RequestMapping("/contact")
@Api(value = "联系人模块", tags = "联系人模块")
public class ContactController {

	private final IContactOuterService contactOuterService;
	private final IWorkStatusService workStatusService;


	/**
	 * 外部联系人
	 */
	@GetMapping("/outer-dept")
	@ApiOperation(value = "外部联系人机构分页", notes = "传入分页参数")
	@ApiOperationSupport(order = 1)
	public R address(Query query) {
		IPage<ContactOuter> page = contactOuterService.selectPage(AuthUtil.getUserId(), Condition.getPage(query));
		return R.data(ContactOuterWrapper.build().pageVO(page));
	}

	/**
	 * 外部联系人分页
	 */
	@GetMapping("/outer-contact")
	@ApiOperation(value = "外部联系人分页", notes = "传入deptId 分页参数")
	@ApiOperationSupport(order = 2)
	public R contactPage(@RequestParam Long deptId, Query query) {
		IPage<ContactOuter> page = contactOuterService.contactPage(deptId, AuthUtil.getUserId(), Condition.getPage(query));
		return R.data(ContactOuterUserWrapper.build().pageVO(page));
	}


	/**
	 * 用户工作状态
	 */
	@GetMapping("/user-workStatus")
	@ApiOperation(value = "用户工作状态", notes = "传入userId")
	@ApiOperationSupport(order = 3)
	public R workStatus(@RequestParam Long userId) {
		WorkStatusVO workStatusVO = workStatusService.workStatus(userId);
		return R.data(workStatusVO);
	}

	/**
	 * 修改用户信息
	 */
	@PostMapping("/update-user")
	@ApiOperation(value = "修改用户信息", notes = "传入")
	@ApiOperationSupport(order = 3)
	public R updateUser(@RequestBody @Valid UpdateUserDTO dto) {
		return R.status(contactOuterService.updateUser(dto));
	}


}
