package com.wtkj.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.wtkj.dto.SearchProjectFileDTO;
import com.wtkj.dto.UploadInnerFileDTO;
import com.wtkj.entity.FileAndFolder;
import com.wtkj.entity.InnerFileAndFolder;
import com.wtkj.service.ICommonService;
import com.wtkj.service.IInnerFileAndFolderService;
import com.wtkj.wrapper.FileAndFolderWrapper;
import com.wtkj.wrapper.InnerFileAndFolderWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Objects;

import static com.wtkj.config.MagicValue.ONE;
import static com.wtkj.config.MagicValue.TWO;

/**
 * @author Blizzard
 * @create at 2023-09-26 09:24
 * @describe
 */
@RestController
@AllArgsConstructor
@RequestMapping("/resource")
@Api(value = "资料库模块", tags = "资料库模块")
public class ResourceController {

	private final IInnerFileAndFolderService innerFileAndFolderService;

	private final ICommonService commonService;

	/**
	 * 内部资料库
	 */
	@PostMapping("/submit-inner-folder")
	@ApiOperation(value = "创建或者修改内部文件夹或者文件", notes = "")
	@ApiOperationSupport(order = 1)
	public R submitFolder(@RequestBody @Valid InnerFileAndFolder folder) {
		LambdaQueryWrapper<InnerFileAndFolder> lqw = new LambdaQueryWrapper<>();
		lqw.eq(InnerFileAndFolder::getParentId, folder.getParentId());
		lqw.eq(InnerFileAndFolder::getTitle, folder.getTitle());
		if (Objects.equals(folder.getType(), ONE)) {
			lqw.eq(InnerFileAndFolder::getType, ONE);
			long cnt = innerFileAndFolderService.count(lqw);
			if (cnt > 0L) {
				throw new ServiceException("此文件夹下已经存在同名文件");
			}
		} else if (Objects.equals(folder.getType(), TWO)) {
			lqw.eq(InnerFileAndFolder::getType, TWO);
			long cnt = innerFileAndFolderService.count(lqw);
			if (cnt > 0L) {
				throw new ServiceException("此文件夹下已经存在同名文件夹");
			}
		}
		return R.status(innerFileAndFolderService.saveOrUpdate(folder));
	}

	/**
	 * 删除文件/文件夹
	 */
	@GetMapping("/remove-inner-file")
	@ApiOperation(value = "删除内部文件或者文件夹", notes = "")
	@ApiOperationSupport(order = 2)
	public R remove(@RequestParam String ids) {
		return R.status(commonService.delete(Func.toLongList(ids)));
	}

	/**
	 * 分页
	 */
	@GetMapping("/inner-page")
	@ApiOperation(value = "内部文件夹 分页", notes = "传入parentId,分页参数")
	@ApiOperationSupport(order = 3)
	public R innerPage(@RequestParam Long deptId, @RequestParam Long parentId, @RequestParam String name, Query query) {
		IPage<InnerFileAndFolder> page = innerFileAndFolderService.selectPage(deptId, parentId, name, Condition.getPage(query));
		return R.data(InnerFileAndFolderWrapper.build().pageVO(page));
	}

	/**
	 * 文件数量
	 */
	@GetMapping("/file-and-page-amount")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "文夹下的文件数量", notes = "传入文件夹主键id")
	public R<JSONObject> getPageAmount(@RequestParam Long deptId, @RequestParam Long parentId) {
		return R.data(innerFileAndFolderService.getPageAndFileAmount(deptId, parentId));
	}

	/**
	 * 上传文件
	 */
	@PostMapping("/upload-inner-file")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "上传内部文件", notes = "传入文件夹主键id")
	public R uploadInnerFile(@RequestBody @Valid List<UploadInnerFileDTO> files) {
		return R.status(commonService.uploadInnerFile(files));
	}


	/**
	 * 项目资料
	 */
	@PostMapping("/search-project-file")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "项目资料", notes = "")
	public R searchProjectFile(@RequestBody SearchProjectFileDTO dto) {
		Query query = new Query();
		query.setSize(dto.getSize());
		query.setCurrent(dto.getCurrent());
		IPage<FileAndFolder> page = commonService.esSearch(dto, Condition.getPage(query));
		return R.data(FileAndFolderWrapper.build().pageVO(page));
	}

	/**
	 * 打开文件
	 */
	@PostMapping("/open-search-file")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "打开搜索结果文件", notes = "")
	public R openSearch(@RequestParam Long projectId, @RequestParam Long fileId, @RequestParam String text) {
		return R.data(commonService.openSearchFile(projectId, fileId, text));
	}

}
