package com.wtkj.handler;

import com.wtkj.entity.Project;
import com.wtkj.entity.Task;
import com.wtkj.service.IProjectService;
import com.wtkj.service.ITaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author Blizzard
 * @create at 2023-10-05 11:47
 * @describe 定时任务扫描任务状态
 */
@Configuration
@Slf4j
public class Schedule {

	@Autowired
	private ITaskService taskService;
	@Autowired
	private IProjectService projectService;

	@Scheduled(cron = "0 0 3 * * ?")
	private void task() {
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd"); // 设置日期格式
		String now = LocalDate.now().format(formatter);
		//今天过期的任务
		List<Task> tasks = taskService.getTodayToExpire(now);
		if (!CollectionUtils.isEmpty(tasks)) {
			tasks.forEach(task -> {
				task.setTaskStatus(6);
				Long projectId = task.getProjectId();
				Project byId = projectService.getById(projectId);
				if (byId != null) {
					byId.setTodoTask(Math.max(byId.getTodoTask() - 1, 0));
					projectService.updateById(byId);
				}
			});
			taskService.updateBatchById(tasks);
		}
	}
}
