package com.wtkj.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.wtkj.entity.InnerFileAndFolder;
import com.wtkj.mapper.InnerFileAndFolderMapper;
import com.wtkj.service.IInnerFileAndFolderService;
import com.wutong.file.feign.IFileClient;
import com.wutong.file.vo.FileVO;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Blizzard
 * @create at 2023-09-26 14:44
 * @describe
 */
@Service
@AllArgsConstructor
public class InnerFileAndFolderServiceImpl extends BaseServiceImpl<InnerFileAndFolderMapper, InnerFileAndFolder> implements IInnerFileAndFolderService {

	private final IFileClient fileClient;

	@Override
	public List<InnerFileAndFolder> getChildren(Long deptId, Long parentId) {
		return getChildrenByFolderId(new ArrayList<>(), deptId, parentId);
	}

	@Override
	public IPage<InnerFileAndFolder> selectPage(Long deptId, Long parentId, String name, IPage<InnerFileAndFolder> page) {
		if (StringUtil.isNotBlank(name)) {
			parentId = null;
		}
		return page.setRecords(baseMapper.getPage(deptId, parentId, name, page));
	}

	@Override
	public JSONObject getPageAndFileAmount(Long deptId, Long parentId) {

		JSONObject result = new JSONObject();
		//文件数量
		Integer fileAmount = this.getFileAmountByFolderId(deptId, parentId);
		result.put("amount", fileAmount);
		//文件
		int totalPage = 0;
		int volume = 0;
		List<InnerFileAndFolder> files = this.getChildrenFilesByFolderId(deptId, parentId);
		if (!CollectionUtils.isEmpty(files)) {
			totalPage = files.stream().filter(f -> f.getBladeFileId() != null).map(f -> fileClient.findImageList(f.getBladeFileId())).filter(rpc -> rpc.isSuccess() && rpc.getData() != null).map(R::getData).mapToInt(List::size).sum();
			volume = files.stream().filter(f -> f.getBladeFileId() != null).map(f -> fileClient.findById(f.getBladeFileId())).filter(rpc -> rpc.isSuccess() && rpc.getData() != null).map(R::getData).map(FileVO::getVolume).mapToInt(Integer::parseInt).sum();
		}
		result.put("totalPage", totalPage);
		result.put("volume", volume);
		int folderAmount = 0;
		List<InnerFileAndFolder> folders = this.getChildrenFolderByFolderId(deptId, parentId);
		if (!CollectionUtils.isEmpty(folders)) {
			folderAmount = folders.size();
		}
		result.put("folderAmount", folderAmount);
		return result;
	}

	@Override
	public Integer getFileAmountByFolderId(Long deptId, Long parentId) {
		//递归子级文件夹 获取文件个数
		return getChildrenFileAmountByFolderId(0, deptId, parentId);
	}

	@Override
	public List<InnerFileAndFolder> getChildrenFilesByFolderId(Long deptId, Long parentId) {
		return getChildrenFileByFolderId(new ArrayList<>(), deptId, parentId);
	}

	@Override
	public List<InnerFileAndFolder> getChildrenFolderByFolderId(Long deptId, Long parentId) {
		return baseMapper.getChildrenFolderList(deptId, parentId);
	}


	private List<InnerFileAndFolder> getChildrenFileByFolderId(ArrayList<InnerFileAndFolder> result, Long deptId, Long parentId) {
		//本级下的文件
		List<InnerFileAndFolder> files = baseMapper.getChildrenFileList(deptId, parentId);
		if (!org.springframework.util.CollectionUtils.isEmpty(files)) {
			result.addAll(files);
		}
		//本级的文件夹
		List<InnerFileAndFolder> childrenFolder = baseMapper.getChildrenFolderList(deptId, parentId);
		//递归出口，子集不为空时，一直去遍历
		if (!com.baomidou.mybatisplus.core.toolkit.CollectionUtils.isEmpty(childrenFolder)) {
			for (InnerFileAndFolder folder : childrenFolder) {
				getChildrenFileByFolderId(result, folder.getDeptId(), folder.getId());
			}
		}
		return result;
	}

	private Integer getChildrenFileAmountByFolderId(int count, Long deptId, Long parentId) {
		//本级下的文件数量
		count = count + baseMapper.getFileAmountByFolderId(deptId, parentId);
		//本级下的文件夹
		List<InnerFileAndFolder> children = baseMapper.getChildrenFolderList(deptId, parentId);
		//递归出口，子集不为空时，一直去遍历
		if (children != null && !children.isEmpty()) {
			for (InnerFileAndFolder child : children) {
				int cnt = getChildrenFileAmountByFolderId(0, child.getDeptId(), child.getId());
				count = cnt + count;
			}
		}
		return count;

	}

	private List<InnerFileAndFolder> getChildrenByFolderId(ArrayList<InnerFileAndFolder> result, Long deptId, Long parentId) {
		//文件下的文件和文件夹
		List<InnerFileAndFolder> children = baseMapper.getChildren(parentId);
		if (!org.springframework.util.CollectionUtils.isEmpty(children)) {
			result.addAll(children);
		}
		//本级的文件夹
		List<InnerFileAndFolder> childrenFolder = baseMapper.getChildrenFolderList(deptId, parentId);
		if (!CollectionUtils.isEmpty(childrenFolder)) {
			for (InnerFileAndFolder folder : childrenFolder) {
				getChildrenByFolderId(result, folder.getDeptId(), folder.getId());
			}
		}
		return result;
	}
}
