package com.wtkj.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.wtkj.dto.ProjectPageDTO;
import com.wtkj.entity.Project;
import com.wtkj.entity.ProjectAuth;
import com.wtkj.entity.Task;
import com.wtkj.entity.TaskContract;
import com.wtkj.mapper.ProjectMapper;
import com.wtkj.service.*;
import com.wtkj.vo.ProjectStatisticsVO;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import static com.wtkj.config.MagicValue.*;

/**
 * @author Blizzard
 * @create at 2023-09-13 16:00
 * @describe
 */
@Service
@AllArgsConstructor
public class ProjectServiceImpl extends BaseServiceImpl<ProjectMapper, Project> implements IProjectService {

	private final IAsyncService asyncService;
	private final IProjectAuthService projectAuthService;
	private final ITaskService taskService;
	private final ITaskContractService taskContractService;

	@Override
	public boolean submit(Project project) {
		boolean flag = false;
		BladeUser user = AuthUtil.getUser();
		if (project.getId() == null) {
			LocalDate date = LocalDate.now();
			project.setYear(String.valueOf(date.getYear()));
			flag = this.save(project);
			asyncService.createProject(project, user);
		} else {
			flag = this.updateById(project);
		}
		return flag;
	}

	@Override
	public IPage<Project> getPage(ProjectPageDTO dto, IPage<Project> page) {
		IPage<Project> projectIPage = new Page<>();
		/*LambdaQueryWrapper<Project> lqw = new LambdaQueryWrapper<>();
		//判断当前登录用户的身份 是业主还是服务机构
		String deptIdStr = AuthUtil.getDeptId();
		Long deptId = Long.valueOf(deptIdStr);
		R<Dept> rpc = sysClient.getDept(deptId);
		if (rpc.isSuccess()) {
			//项目名称
			if (StringUtil.isNotBlank(dto.getName())) {
				lqw.eq(Project::getName, dto.getName());
			}
			//区域
			if (StringUtil.isNotBlank(dto.getAreaCode())) {
				lqw.eq(Project::getAreaCode, dto.getAreaCode());
			}
			Integer deptCategory = rpc.getData().getDeptCategory();
			//如果是业主 即查询主管单位是该deptId的项目
			if (deptCategory != null && deptCategory.equals(3)) {
				//主管单位 业主自己
				lqw.eq(Project::getCompetentUnit, deptId);
				//协作单位
				lqw.like(StringUtil.isNotBlank(dto.getCooperationUint()), Project::getCooperationUint, dto.getCooperationUint());
			} else {
				//主管单位
				if (dto.getCompetentUnit() != null) {
					lqw.eq(Project::getCompetentUnit, dto.getCompetentUnit());
				}
				//如果不是业主 ---> 判断是否是服务商的管理员  服务商管理员可以查看所有的项目
				String userRole = AuthUtil.getUserRole();
				if (userRole != null && userRole.contains(STAFF_ADMIN)) {
					List<ProjectAuth> auths = projectAuthService.getByTopDept(dto.getTopDept(), null);
					if (!CollectionUtils.isEmpty(auths)) {
						Set<Long> projectIds = auths.stream().map(ProjectAuth::getProjectId).collect(Collectors.toSet());
						lqw.in(Project::getId, projectIds);
					}
				} else {
					//不是服务机构管理员 则会是项目经理或者任务执行人  从项目权限表里取
					List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(deptId, AuthUtil.getUserId());
					if (!CollectionUtils.isEmpty(auths)) {
						Set<Long> projectIds = auths.stream().map(ProjectAuth::getProjectId).collect(Collectors.toSet());
						lqw.in(Project::getId, projectIds);
					}
				}
			}
		}
		return this.page(page, lqw);*/
		//1.机构类型
		Integer category = dto.getDeptCategory();
		Long topDept = dto.getTopDept();
		Set<Long> projectIds = new HashSet<>();
		if (category != null) {
			if (category.equals(3)) {
				//1.1 是业主 ---> 项目权限表里 topDept 字段是自己的集合
				List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, null, null);
				if (!CollectionUtils.isEmpty(auths)) {
					Set<Long> ids = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
					projectIds.addAll(ids);
				}
			} else if (category.equals(4)) {
				String role = AuthUtil.getUserRole();
				Long deptId = Func.firstLong(AuthUtil.getDeptId());
				Long userId = AuthUtil.getUserId();
				//1.2 是服务机构
				if (role != null) {
					if (role.contains(STAFF_ADMIN)) {
						// 1.2.1 机构管理员 ---> 项目权限表里 topDept 字段是自己的集合
						List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, null, null);
						if (!CollectionUtils.isEmpty(auths)) {
							Set<Long> ids = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
							projectIds.addAll(ids);
						}
					} else if (role.contains(STAFF_SECOND_ADMIN)) {
						// 1.2.2 次级机构主管  ---> 项目权限表里 userDept 字段是自己的集合
						List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, deptId, null);
						if (!CollectionUtils.isEmpty(auths)) {
							Set<Long> ids = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
							projectIds.addAll(ids);
						}
					} else if (role.contains(STAFF_USER) || role.contains(STAFF_PROJECT_MANAGER)) {
						//1.2.3 普通员工和项目经理   --->项目权限表里 userId 是自己的
						List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, deptId, userId);
						if (!CollectionUtils.isEmpty(auths)) {
							Set<Long> ids = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
							projectIds.addAll(ids);
						}
					}
				}
			}
		}
		if (!CollectionUtils.isEmpty(projectIds)) {
			LambdaQueryWrapper<Project> lqw = new LambdaQueryWrapper<>();
			//年份
			lqw.eq(dto.getYear() != null, Project::getYear, dto.getYear());
			//项目名称
			if (StringUtil.isNotBlank(dto.getName())) {
				lqw.like(Project::getName, dto.getName());
			}
			//区域
			if (StringUtil.isNotBlank(dto.getAreaCode())) {
				lqw.eq(Project::getAreaCode, dto.getAreaCode());
			}
			//协作单位
			lqw.like(StringUtil.isNotBlank(dto.getCooperationUint()), Project::getCooperationUint, dto.getCooperationUint());
			//主管单位
			lqw.eq(dto.getCompetentUnit() != null, Project::getCompetentUnit, dto.getCompetentUnit());

			lqw.in(Project::getId, projectIds);

			//任务数量降序
			if (dto.getTodoAsc().equals(1)) {
				lqw.orderByDesc(Project::getTodoTask);
			} else {
				lqw.orderByAsc(Project::getTodoTask);
			}
			// todo 截至时间降序
			projectIPage = this.page(page, lqw);
		}
		return projectIPage;
	}

	@Override
	public Set<Long> getByCreate(Long createDept, Long createUser) {
		return baseMapper.selectByCreate(createDept, createUser);
	}

	@Override
	public ProjectStatisticsVO projectStatistics(Long topDept, Integer deptCategory, String year) {
		ProjectStatisticsVO vo = new ProjectStatisticsVO();
		if (deptCategory != null) {
			if (deptCategory.equals(3)) {
				//是业主 --> 权限表里 topDept是参数topDept
				List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, null, null);
				if (!CollectionUtils.isEmpty(auths)) {
					Set<Long> ids = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
					LambdaQueryWrapper<Project> lqw = new LambdaQueryWrapper<>();
					lqw.eq(Project::getYear, year);
					lqw.in(Project::getId, ids);
					List<Project> list = this.list(lqw);
					if (!CollectionUtils.isEmpty(list)) {
						Set<Long> projectIds = list.stream().map(Project::getId).collect(Collectors.toSet());
						//项目总数
						vo.setProjectCount(list.size());
						//项目总投
						BigDecimal amount = list.stream().map(Project::getTotalAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
						vo.setTotalAmount(amount);
						//服务合同金额 --> 所有业主的项目的任务下的所有合同
						List<TaskContract> contracts = taskContractService.getByProjectIds(projectIds);
						if (!CollectionUtils.isEmpty(contracts)) {
							BigDecimal reduce = contracts.stream().map(TaskContract::getAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
							vo.setContractAmount(reduce);
						}
						//待完成任务总数
						List<Task> tasks = taskService.getTodoByYear(projectIds, year);
						if (!CollectionUtils.isEmpty(tasks)) {
							vo.setTodoTask(tasks.size());
						}
					}
				}
			} else if (deptCategory.equals(4)) {
				String role = AuthUtil.getUserRole();
				Long deptId = Func.firstLong(AuthUtil.getDeptId());
				Long userId = AuthUtil.getUserId();
				if (role != null) {
					if (role.contains(STAFF_ADMIN)) {
						// 1.2.1 机构管理员 ---> 项目权限表里 topDept 字段是自己的集合
						List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, null, null);
						if (!CollectionUtils.isEmpty(auths)) {
							Set<Long> projectIds = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
							LambdaQueryWrapper<Project> lqw = new LambdaQueryWrapper<>();
							lqw.eq(Project::getYear, year);
							lqw.in(Project::getId, projectIds);
							List<Project> projects = this.list(lqw);
							if (!CollectionUtils.isEmpty(projects)) {
								//项目总数
								vo.setProjectCount(projects.size());
								//项目总投资
								BigDecimal amount = projects.stream().map(Project::getTotalAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
								vo.setTotalAmount(amount);
								//服务合同金额 ---> 合同乙方是参数topDept
								List<TaskContract> contracts = taskContractService.getByDeptIdAndYear(topDept, Integer.valueOf(year), null);
								if (!CollectionUtils.isEmpty(contracts)) {
									BigDecimal reduce = contracts.stream().map(TaskContract::getAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
									vo.setContractAmount(reduce);
								}
								//待完成任务数
								List<Task> tasks = taskService.getByDeptAndDate(topDept, null, null, null);
								if (!CollectionUtils.isEmpty(tasks)) {
									Set<Task> collect = tasks.stream().filter(f -> f.getYear() != null && f.getYear().equals(Integer.valueOf(year)) && f.getTaskStatus() != 6 && f.getTaskStatus() != 5 && f.getTaskStatus() != 4).collect(Collectors.toSet());
									if (!CollectionUtils.isEmpty(collect)) {
										vo.setTodoTask(collect.size());
									}
								}
							}
						}
					} else if (role.contains(STAFF_SECOND_ADMIN)) {
						// 1.2.2 次级机构主管  ---> 项目权限表里 userDept 字段是自己的集合
						List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, deptId, null);
						if (!CollectionUtils.isEmpty(auths)) {
							Set<Long> projectIds = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
							LambdaQueryWrapper<Project> lqw = new LambdaQueryWrapper<>();
							lqw.eq(Project::getYear, year);
							lqw.in(Project::getId, projectIds);
							List<Project> projects = this.list(lqw);
							if (!CollectionUtils.isEmpty(projects)) {
								//项目总数
								vo.setProjectCount(projects.size());
								//项目总投资
								BigDecimal amount = projects.stream().map(Project::getTotalAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
								vo.setTotalAmount(amount);
								//服务合同金额  --> 任务执行单位是dept 的合同
								List<Task> tasks = taskService.getByDeptAndDate(topDept, deptId, null, null);
								if (!CollectionUtils.isEmpty(tasks)) {
									Set<Long> taskIds = tasks.stream().map(Task::getId).collect(Collectors.toSet());
									List<TaskContract> contracts = taskContractService.getByDeptIdAndYear(topDept, Integer.valueOf(year), taskIds);
									if (!CollectionUtils.isEmpty(contracts)) {
										BigDecimal reduce = contracts.stream().map(TaskContract::getAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
										vo.setContractAmount(reduce);
									}
									//待完成任务总数
									Set<Task> collect = tasks.stream().filter(f -> f.getYear() != null && f.getYear().equals(Integer.valueOf(year)) && f.getTaskStatus() != 6 && f.getTaskStatus() != 5 && f.getTaskStatus() != 4).collect(Collectors.toSet());
									if (!CollectionUtils.isEmpty(collect)) {
										vo.setTodoTask(collect.size());
									}
								}
							}
						}
					} else if (role.contains(STAFF_USER) || role.contains(STAFF_PROJECT_MANAGER)) {
						//1.2.3 普通员工和项目经理   --->项目权限表里 userId 是自己的
						List<ProjectAuth> auths = projectAuthService.getByDeptAndUser(topDept, deptId, userId);
						if (!CollectionUtils.isEmpty(auths)) {
							Set<Long> projectIds = auths.stream().map(ProjectAuth::getProjectId).filter(Objects::nonNull).collect(Collectors.toSet());
							LambdaQueryWrapper<Project> lqw = new LambdaQueryWrapper<>();
							lqw.eq(Project::getYear, year);
							lqw.in(Project::getId, projectIds);
							List<Project> projects = this.list(lqw);
							if (!CollectionUtils.isEmpty(projects)) {
								//项目总数
								vo.setProjectCount(projects.size());
								//项目总投资
								BigDecimal amount = projects.stream().map(Project::getTotalAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
								vo.setTotalAmount(amount);
								//服务合同金额  --> 任务执行者是userid
								List<Task> tasks = taskService.getByDeptAndDate(topDept, deptId, String.valueOf(userId), null);
								if (!CollectionUtils.isEmpty(tasks)) {
									Set<Long> taskIds = tasks.stream().map(Task::getId).collect(Collectors.toSet());
									List<TaskContract> contracts = taskContractService.getByDeptIdAndYear(topDept, Integer.valueOf(year), taskIds);
									if (!CollectionUtils.isEmpty(contracts)) {
										BigDecimal reduce = contracts.stream().map(TaskContract::getAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
										vo.setContractAmount(reduce);
									}
									//待完成任务总数
									Set<Task> collect = tasks.stream().filter(f -> f.getYear() != null && f.getYear().equals(Integer.valueOf(year)) && f.getTaskStatus() != 6 && f.getTaskStatus() != 5 && f.getTaskStatus() != 4).collect(Collectors.toSet());
									if (!CollectionUtils.isEmpty(collect)) {
										vo.setTodoTask(collect.size());
									}
								}
							}
						}
					}
				}
			}
		}
		return vo;
	}


	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean delete(List<Long> projectIds) {
		LambdaQueryWrapper<Task> lqw = new LambdaQueryWrapper<>();
		lqw.in(Task::getProjectId, projectIds);
		List<Task> list = taskService.list(lqw);
		if (!CollectionUtils.isEmpty(list)) {
			taskService.removeBatchByIds(list);
		}
		LambdaQueryWrapper<ProjectAuth> wrapper = new LambdaQueryWrapper<>();
		wrapper.in(ProjectAuth::getProjectId, projectIds);
		List<ProjectAuth> auths = projectAuthService.list(wrapper);
		if (!CollectionUtils.isEmpty(auths)) {
			projectAuthService.removeByIds(auths);
		}
		return removeByIds(projectIds);
	}

}
