package com.wtkj.wrapper;

import com.wtkj.entity.ProjectGroup;
import com.wtkj.service.IProjectGroupService;
import com.wtkj.vo.ProjectGroupVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Blizzard
 * @create at 2023-09-20 09:29
 * @describe
 */
public class ProjectGroupWrapper extends BaseEntityWrapper<ProjectGroup, ProjectGroupVO> {

	private static ISysClient sysClient;

	private static IUserClient userClient;

	private static IProjectGroupService projectGroupService;

	public static ProjectGroupWrapper build() {
		sysClient = SpringUtil.getBean(ISysClient.class);
		userClient = SpringUtil.getBean(IUserClient.class);
		projectGroupService = SpringUtil.getBean(IProjectGroupService.class);
		return new ProjectGroupWrapper();
	}

	@Override
	public ProjectGroupVO entityVO(ProjectGroup entity) {
		ProjectGroupVO vo = Objects.requireNonNull(BeanUtil.copy(entity, ProjectGroupVO.class));
		Long projectId = entity.getProjectId();
		//机构名称
		Long beInvitedDept = entity.getBeInvitedDept();
		R<Dept> dept = sysClient.getDept(beInvitedDept);
		if (dept.isSuccess()) {
			vo.setDeptName(dept.getData().getDeptName());
		}
		//机构下的用户
		List<ProjectGroup> groups = projectGroupService.getUserByBeInvitedDept(projectId, beInvitedDept);
		if (!CollectionUtils.isEmpty(groups)) {
			Set<Long> userIds = groups.stream().map(ProjectGroup::getUserId).collect(Collectors.toSet());
			List<User> users = new ArrayList<>();
			userIds.forEach(userId -> {
				R<User> rpc = userClient.userInfoById(userId);
				if (rpc.isSuccess()) {
					users.add(rpc.getData());
				}
			});
			vo.setSize(users.size());
			vo.setUsers(users);
		}
		return vo;
	}
}
