package com.wtkj.wrapper;

import com.wtkj.entity.Project;
import com.wtkj.entity.ProjectStage;
import com.wtkj.service.IProjectStageService;
import com.wtkj.vo.ProjectVO;
import com.wutong.file.feign.IFileClient;
import com.wutong.file.vo.FileVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;

import java.util.Objects;

/**
 * @author Blizzard
 * @create at 2023-09-16 23:08
 * @describe
 */
public class ProjectWrapper extends BaseEntityWrapper<Project, ProjectVO> {

	private static IProjectStageService stageService;
	private static IFileClient fileClient;

	private static ISysClient sysClient;


	public static ProjectWrapper build() {
		stageService = SpringUtil.getBean(IProjectStageService.class);
		fileClient = SpringUtil.getBean(IFileClient.class);
		sysClient = SpringUtil.getBean(ISysClient.class);
		return new ProjectWrapper();
	}

	@Override
	public ProjectVO entityVO(Project entity) {
		ProjectVO vo = Objects.requireNonNull(BeanUtil.copy(entity, ProjectVO.class));
		Long currentStage = entity.getCurrentStage();
		if (currentStage != null) {
			ProjectStage byId = stageService.getById(currentStage);
			if (byId != null) {
				vo.setCurrentStageName(byId.getName());
			}
		}
		Long imgId = entity.getImgId();
		if (imgId != null) {
			R<FileVO> byId = fileClient.findById(imgId);
			if (byId.isSuccess()) {
				vo.setImgUrl(byId.getData().getFilePath());
			}
		}
		Long competentUnit = entity.getCompetentUnit();
		if (competentUnit != null) {
			R<Dept> dept = sysClient.getDept(competentUnit);
			if (dept.isSuccess()) {
				vo.setCompetentUnitName(dept.getData().getDeptName());
			}
		}
		return vo;
	}
}
