package com.wtkj.wrapper;

import com.wtkj.entity.*;
import com.wtkj.service.IFileAndFolderService;
import com.wtkj.service.IFileReadService;
import com.wtkj.service.IProjectService;
import com.wtkj.service.ITaskFileService;
import com.wtkj.vo.TaskVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.entity.DictBiz;
import org.springblade.system.feign.IDictBizClient;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Blizzard
 * @create at 2023-09-15 16:18
 * @describe
 */
public class TaskWrapper extends BaseEntityWrapper<Task, TaskVO> {

	private static IUserClient userClient;
	private static ISysClient sysClient;
	private static IFileAndFolderService fileAndFolderService;
	private static IDictBizClient dictBizClient;
	private static IProjectService projectService;
	private static IFileReadService fileReadService;
	private static ITaskFileService taskFileService;


	public static TaskWrapper build() {
		userClient = SpringUtil.getBean(IUserClient.class);
		sysClient = SpringUtil.getBean(ISysClient.class);
		fileAndFolderService = SpringUtil.getBean(IFileAndFolderService.class);
		dictBizClient = SpringUtil.getBean(IDictBizClient.class);
		projectService = SpringUtil.getBean(IProjectService.class);
		fileReadService = SpringUtil.getBean(IFileReadService.class);
		taskFileService = SpringUtil.getBean(ITaskFileService.class);
		return new TaskWrapper();
	}

	@Override
	public TaskVO entityVO(Task entity) {
		TaskVO vo = Objects.requireNonNull(BeanUtil.copy(entity, TaskVO.class));
		Long projectId = entity.getProjectId();
		if (projectId != null) {
			Project byId = projectService.getById(projectId);
			if (byId != null) {
				vo.setProjectName(byId.getName());
			}
		}
		//项目经理
		Long projectManager = vo.getProjectManager();
		if (projectManager != null) {
			R<User> userR = userClient.userInfoById(projectManager);
			if (userR.isSuccess()) {
				vo.setProjectManagerUser(userR.getData().getName());
			}
		}
		//协作单位
		Long orgDeptId = vo.getOrgDeptId();
		if (orgDeptId != null) {
			R<Dept> rpc = sysClient.getDept(orgDeptId);
			if (rpc.isSuccess()) {
				vo.setDispatchUintName(rpc.getData().getDeptName());
			}
		}
		//执行机构
		Long executeDept = vo.getExecuteDept();
		if (executeDept != null) {
			R<Dept> dept = sysClient.getDept(executeDept);
			if (dept.isSuccess()) {
				vo.setExecuteDeptName(dept.getData().getDeptName());
			}
		}

		//任务创建人
		Long createUser = entity.getCreateUser();
		R<User> rpc = userClient.userInfoById(createUser);
		if (rpc.isSuccess()) {
			vo.setCreateUserName(rpc.getData().getName());
		}

		//审核人
		Long first = entity.getFirstCheckUser();
		Long second = entity.getSecondCheckUser();
		if (first != null) {
			R<User> userR = userClient.userInfoById(first);
			if (userR.isSuccess()) {
				vo.setFirstUser(userR.getData());
			}
		}
		if (second != null) {
			R<User> userR = userClient.userInfoById(second);
			if (userR.isSuccess()) {
				vo.setSecondUser(userR.getData());
			}
		}
		//审核过的用户
		//1.成果文件
		List<Long> users = new ArrayList<>();
		List<TaskFile> resultFiles = taskFileService.fileList(entity.getId());
		if (!CollectionUtils.isEmpty(resultFiles)) {
			//文件ids
			Set<Long> fileIds = resultFiles.stream().map(TaskFile::getBladeFileId).collect(Collectors.toSet());
			List<FileRead> reads = fileReadService.getByFileIds(fileIds);
			if (!CollectionUtils.isEmpty(reads)) {
				Set<Long> userIds = reads.stream().map(FileRead::getUserId).collect(Collectors.toSet());
				if (userIds.contains(first)) {
					users.add(first);
				}
				if (userIds.contains(second)) {
					users.add(second);
				}
			}
		}
		vo.setCheckUserIds(users);

		//任务执行人
		String executeUser = entity.getExecuteUser();
		if (executeUser != null) {
			List<String> execUsers = new ArrayList<>();
			List<Long> userIds = Func.toLongList(executeUser);
			userIds.forEach(userId -> {
				R<User> userR = userClient.userInfoById(userId);
				if (userR.isSuccess()) {
					execUsers.add(userR.getData().getName());
				}
			});
			vo.setExecuteUsers(execUsers);
		}

		//任务相关联文件
		String relatedIds = entity.getRelatedIds();
		if (StringUtil.isNotBlank(relatedIds)) {
			List<FileAndFolder> files = new ArrayList<>();
			List<Long> fileIds = Func.toLongList(relatedIds);
			fileIds.forEach(fileId -> {
				FileAndFolder byId = fileAndFolderService.getById(fileId);
				if (byId != null) {
					files.add(byId);
				}
			});
			vo.setFiles(files);
		}
		//任务标签的数组
		String tags = entity.getTags();
		String category = entity.getCategory();
		List<String> dictBizs = new ArrayList<>();
		if (StringUtil.isNotBlank(tags)) {
			List<Long> intList = Func.toLongList(tags);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		if (StringUtil.isNotBlank(category)) {
			List<Long> intList = Func.toLongList(category);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		vo.setTagsArray(dictBizs);
		return vo;
	}
}
